@@ -279,7 +279,7 @@ namespace acommon {
     {
       if (conv) {
         buf.clear();
-        conv->convert(str, strlen(str), buf, buf0);
+        conv->convert(str, -1, buf, buf0);
         buf.ensure_null_end();
         return buf.data();
       } else {
@@ -290,7 +290,7 @@ namespace acommon {
     {
       if (conv) {
         buf.clear();
-        conv->convert(str, str.size(), buf, buf0);
+        conv->convert(str, -1, buf, buf0);
         buf.ensure_null_end();
         return buf.data();
       } else {
@@ -341,7 +341,6 @@ namespace acommon {
       conv = pe.data;
       return no_err;
     }
-    // assumes str is null terminated
     PosibErr<char *> operator() (char * str, size_t sz)
     {
       if (conv) {
@@ -361,7 +360,7 @@ namespace acommon {
     {
       if (conv) {
         buf.clear();
-        RET_ON_ERR(conv->convert_ec(str, strlen(str), buf, buf0, str));
+        RET_ON_ERR(conv->convert_ec(str, -1, buf, buf0, str));
         buf.ensure_null_end();
         return buf.data();
       } else {
@@ -372,7 +371,7 @@ namespace acommon {
     {
       if (conv) {
         buf.clear();
-        RET_ON_ERR(conv->convert_ec(str, str.size(), buf, buf0, str));
+        RET_ON_ERR(conv->convert_ec(str, -1, buf, buf0, str));
         buf.ensure_null_end();
         return buf.data();
       } else {
