@@ -12,9 +12,8 @@
 #include "filter_char.hpp"
 #include "posib_err.hpp"
 #include "vector.hpp"
-#include "string_pair_enumeration.hpp"
-
 #include <stdio.h>
+#include <cstdio>
 
 namespace acommon {
 
@@ -22,46 +21,52 @@ namespace acommon {
   class Speller;
   class IndividualFilter;
   class StringList;
+  class Notifier;
   struct ConfigModule;
 
   class Filter : public CanHaveError {
   public:
     
-    bool empty() const {return filters_.empty();}
+    enum FilterType {
+      DECODER=-1,
+      FILTER=0,
+      ENCODER=1
+    };
+
+    bool empty() const { return (filters_.empty() && 
+                                decoders_.empty() && 
+                                encoders_.empty());} 
     void clear();
     void reset();
+    void decode(FilterChar * & start, FilterChar * & stop);
     void process(FilterChar * & start, FilterChar * & stop);
-    void add_filter(IndividualFilter * filter);
+    void encode(FilterChar * & start, FilterChar * & stop);
+    void add_filter(IndividualFilter * filter,void* handles=NULL,
+                    int type=FILTER);
     // setup the filter where the string list is the list of 
     // filters to use.
     Filter();
     ~Filter();
-
- private:
+  private:
     typedef Vector<IndividualFilter *> Filters;
+    Filters decoders_;
     Filters filters_;
+    Filters encoders_;
+    typedef Vector<void*> Handled;
+    Handled handledDecoder;
+    Handled handledFilter;
+    Handled handledEncoder;
   };
 
-  PosibErr<void> set_mode_from_extension(Config * config,
-                                         ParmString filename,
-                                         FILE * in = NULL);
+  void set_mode_from_extension(Config * config,
+			       ParmString filename);
   
   PosibErr<void> setup_filter(Filter &, Config *, 
 			      bool use_decoder, 
 			      bool use_filter, 
 			      bool use_encoder);
   void activate_dynamic_filteroptions(Config *c);
-  void activate_filter_modes(Config * config);
 
-  void load_all_filters(Config * config);
-
-  PosibErr<bool> verify_version(const char * relOp, 
-                                const char * actual, const char * required);
-  PosibErr<void> check_version(const char * requirement);
-
-  PosibErr<StringPairEnumeration *> available_filters(Config *);
-  PosibErr<StringPairEnumeration *> available_filter_modes(Config *);
-
-};
+}
 
 #endif /* ASPELL_FILTER__HPP */
