@@ -8,7 +8,6 @@
 #include <assert.h>
 #include <string.h>
 #include <ctype.h>
-#include <dirent.h>
 
 // POSIX includes
 #ifdef __bsdi__
@@ -18,6 +17,8 @@
 #ifdef WIN32
 #  include <windows.h>
 #  include <winbase.h>
+#else
+#  include <dirent.h>
 #endif
 
 #include "iostream.hpp"
@@ -35,9 +36,6 @@
 #include "stack_ptr.hpp"
 #include "strtonum.hpp"
 #include "lock.hpp"
-#include "string_map.hpp"
-
-#include "gettext.h"
 
 namespace acommon {
 
@@ -74,18 +72,15 @@ namespace acommon {
   // this is in an invalid state if some of the lists
   // has data but others don't
   {
-    StringList key;
     StringList for_dirs;
     ModuleInfoList module_info_list;
     StringList dict_dirs;
     DictExtList    dict_exts;
     DictInfoList   dict_info_list;
-    StringMap      dict_aliases;
     void clear();
     PosibErr<void> fill(Config *, StringList &);
     bool has_data() const {return module_info_list.head_ != 0;}
     void fill_helper_lists(const StringList &);
-    PosibErr<void> fill_dict_aliases(Config *);
   };
 
   class MDInfoListofLists
@@ -229,7 +224,7 @@ namespace acommon {
 	      to_add->c_struct.order_num < 1)) 
 	  {
 	    err.prim_err(bad_value, d.key, d.value,
-			 _("a number between 0 and 1"));
+			 "a number between 0 and 1");
 	    goto RETURN_ERROR;
 	  }
       } else if (d.key == "lib-dir") {
@@ -284,7 +279,7 @@ namespace acommon {
     DictInfoNode(DictInfoNode * n = 0) : next(n) {}
     String name;
     String code;
-    String variety;
+    String jargon;
     String size_str;
     String info_file;
     bool direct;
@@ -301,42 +296,10 @@ namespace acommon {
     }
   }
 
-  const DictExt * find_dict_ext(const DictExtList & l, ParmStr name)
-  {
-    DictExtList::const_iterator   i = l.begin();
-    DictExtList::const_iterator end = l.end();
-    for (; i != end; ++i) 
-    {
-      if (i->ext_size <= name.size() 
-          && strncmp(name + (name.size() - i->ext_size),
-                     i->ext, i->ext_size) == 0)
-        break;
-    }
-    
-    if (i == end) // does not end in one of the extensions in list
-      return 0;
-    else
-      return &*i;
-  }
-
-
   PosibErr<void> DictInfoList::fill(MDInfoListAll & list_all,
 				    Config * config)
   {
-    StringList aliases;
-    config->retrieve_list("dict-alias", &aliases);
-    StringListEnumeration els = aliases.elements_obj();
-    const char * str;
-    while ( (str = els.next()) != 0) {
-      const char * end = strchr(str, ' ');
-      assert(end != 0); // FIXME: Return error
-      String name(str, end - str);
-      RET_ON_ERR(proc_file(list_all, config,
-                           0, name.str(), name.size(), 
-                           find_dict_ext(list_all.dict_exts, ".alias")->module));
-    }
-
-    els = list_all.dict_dirs.elements_obj();
+    StringListEnumeration els = list_all.dict_dirs.elements_obj();
     const char * dir;
     while ( (dir = els.next()) != 0) {
       Dir d(opendir(dir));
@@ -347,9 +310,17 @@ namespace acommon {
 	const char * name = entry->d_name;
 	unsigned int name_size = strlen(name);
 
-	const DictExt * i = find_dict_ext(list_all.dict_exts, 
-                                          ParmString(name, name_size));
-	if (i == 0) // does not end in one of the extensions in list
+	DictExtList::const_iterator   i = list_all.dict_exts.begin();
+	DictExtList::const_iterator end = list_all.dict_exts.end();
+	for (; i != end; ++i) 
+	{
+	  if (i->ext_size < name_size 
+	      && strncmp(name + (name_size - i->ext_size),
+			 i->ext, i->ext_size) == 0)
+	    break;
+	}
+
+	if (i == end) // does not end in one of the extensions in list
 	  continue;
 
 	name_size -= i->ext_size;
@@ -390,23 +361,28 @@ namespace acommon {
     to_add->code.assign(name, p0-name);
     to_add->c_struct.code = to_add->code.c_str();
 
-    // check if the code is in a valid form and normalize entry.  
-    // If its not in a valid form, then ignore this entry
+    // check if the code is in a valid form and normilize entry.  
+    // If its not in a valid form than ignore this entry
 
     if (to_add->code.size() >= 2 
 	&& asc_isalpha(to_add->code[0]) && asc_isalpha(to_add->code[1])) 
     {
-      unsigned s = strcspn(to_add->code.str(), "_");
-      if (s > 3) return no_err;
-      unsigned i = 0;
-      for (; i != s; ++i)
-        to_add->name[i] = to_add->code[i] = asc_tolower(to_add->code[i]);
-      i++;
-      for (; i < to_add->code.size(); ++i)
-        to_add->name[i] = to_add->code[i] = asc_toupper(to_add->code[i]);
-    } else {
+      to_add->name[0] = asc_tolower(to_add->name[0]);
+      to_add->name[1] = asc_tolower(to_add->name[1]);
+      to_add->code[0] = asc_tolower(to_add->code[0]);
+      to_add->code[1] = asc_tolower(to_add->code[1]);
+      if (to_add->code.size() == 2); // do nothing
+      else if (to_add->code.size() == 5 && to_add->code[2] == '_' 
+	       && asc_isalpha(to_add->code[3]) 
+	       && asc_isalpha(to_add->code[4])) {
+	to_add->name[3] = asc_toupper(to_add->name[3]);
+	to_add->name[4] = asc_toupper(to_add->name[4]);
+	to_add->code[3] = asc_toupper(to_add->code[3]);
+	to_add->code[4] = asc_toupper(to_add->code[4]);
+      } else
+	return no_err;
+    } else
       return no_err;
-    }
     
     // Need to do it here as module is about to get a value
     // if it is null
@@ -422,8 +398,8 @@ namespace acommon {
     to_add->c_struct.module = module;
   
     if (p0 + 1 < p1)
-      to_add->variety.assign(p0+1, p1 - p0 - 1);
-    to_add->c_struct.variety = to_add->variety.c_str();
+      to_add->jargon.assign(p0+1, p1 - p0 - 1);
+    to_add->c_struct.jargon = to_add->jargon.c_str();
   
     if (p1 != p2) 
       to_add->size_str.assign(p1, 2);
@@ -432,10 +408,8 @@ namespace acommon {
     to_add->c_struct.size_str = to_add->size_str.c_str();
     to_add->c_struct.size = atoi(to_add->c_struct.size_str);
 
-    if (dir) {
-      to_add->info_file  = dir;
-      to_add->info_file += '/';
-    }
+    to_add->info_file  = dir;
+    to_add->info_file += '/';
     to_add->info_file += name;
   
     while (*prev != 0 && *(DictInfoNode *)*prev < *to_add)
@@ -453,7 +427,7 @@ namespace acommon {
     int res = strcmp(rhs.code, lhs.code);
     if (res < 0) return true;
     if (res > 0) return false;
-    res = strcmp(rhs.variety,lhs.variety);
+    res = strcmp(rhs.jargon,lhs.jargon);
     if (res < 0) return true;
     if (res > 0) return false;
     if (rhs.size < lhs.size) return true;
@@ -513,14 +487,10 @@ namespace acommon {
     dict_info_list.clear();
   }
 
-  PosibErr<void> MDInfoListAll::fill(Config * c, 
-                                     StringList & dirs)
+  PosibErr<void> MDInfoListAll::fill(Config * c, StringList & dirs)
   {
     PosibErr<void> err;
 
-    err = fill_dict_aliases(c);
-    if (err.has_err()) goto RETURN_ERROR;
-
     for_dirs = dirs;
     err = module_info_list.fill(*this, c);
     if (err.has_err()) goto RETURN_ERROR;
@@ -557,24 +527,6 @@ namespace acommon {
     }
   }
 
-  PosibErr<void> MDInfoListAll::fill_dict_aliases(Config * c)
-  {
-    StringList aliases;
-    c->retrieve_list("dict-alias", &aliases);
-    StringListEnumeration els = aliases.elements_obj();
-    const char * str;
-    while ( (str = els.next()) != 0) {
-      const char * end = strchr(str, ' ');
-      if (!end) return make_err(bad_value, "dict-alias", str, 
-                                _("in the form \"<name> <value>\""));
-      String name(str, end - str);
-      while (asc_isspace(*end)) ++end;
-      dict_aliases.insert(name.str(), end);
-    }
-    return no_err;
-  }
-
-
   MDInfoListofLists::MDInfoListofLists()
     : data(0), offset(0), size(0)
   {
@@ -596,10 +548,10 @@ namespace acommon {
     }
   }
 
-  int MDInfoListofLists::find(const StringList & key)
+  int MDInfoListofLists::find(const StringList & dirs)
   {
     for (int i = 0; i != size; ++i) {
-      if (data[i].key == key)
+      if (data[i].for_dirs == dirs)
 	return i + offset;
     }
     return -1;
@@ -612,13 +564,9 @@ namespace acommon {
     Config * config = (Config *)c; // FIXME: WHY?
     int & pos = config->md_info_list_index;
     StringList dirs;
-    StringList key;
     if (!valid_pos(pos)) {
       get_data_dirs(config, dirs);
-      key = dirs;
-      key.add("////////");
-      config->retrieve_list("dict-alias", &key);
-      pos = find(key);
+      pos = find(dirs);
     }
     if (!valid_pos(pos)) {
       MDInfoListAll * new_data = new MDInfoListAll[size + 1];
@@ -634,7 +582,6 @@ namespace acommon {
     if (list_all.has_data())
       return &list_all;
 
-    list_all.key = key;
     RET_ON_ERR(list_all.fill(config, dirs));
 
     return &list_all;
@@ -670,17 +617,18 @@ namespace acommon {
 
   /////////////////////////////////////////////////////////////////
   //
-  // user visible functions and enumeration impl
+  // user visable functions and enumeration impl
   //
 
   //
   // ModuleInfo
   //
 
-  PosibErr<const ModuleInfoList *> get_module_info_list(Config * c)
+  const ModuleInfoList * get_module_info_list(Config * c)
   {
-    RET_ON_ERR_SET(md_info_list_of_lists.get_lists(c), const MDInfoListAll *, la);
-    return &la->module_info_list;
+    const MDInfoListAll * la = md_info_list_of_lists.get_lists(c);
+    if (la == 0) return 0;
+    else return &la->module_info_list;
   }
 
   ModuleInfoEnumeration * ModuleInfoList::elements() const
@@ -725,16 +673,11 @@ namespace acommon {
   // DictInfo
   //
 
-  PosibErr<const DictInfoList *> get_dict_info_list(Config * c)
+  const DictInfoList * get_dict_info_list(Config * c)
   {
-    RET_ON_ERR_SET(md_info_list_of_lists.get_lists(c), const MDInfoListAll *, la);
-    return &la->dict_info_list;
-  }
-
-  PosibErr<const StringMap *> get_dict_aliases(Config * c)
-  {
-    RET_ON_ERR_SET(md_info_list_of_lists.get_lists(c), const MDInfoListAll *, la);
-    return &la->dict_aliases;
+    const MDInfoListAll * la = md_info_list_of_lists.get_lists(c);
+    if (la == 0) return 0;
+    else return &la->dict_info_list;
   }
 
   DictInfoEnumeration * DictInfoList::elements() const
