@@ -30,9 +30,6 @@
 // const char * s2 = "...";
 // foo(s2);
 //
-// The string is expected to be null terminated, even if size is give
-// during construction.
-//
 
 namespace acommon {
 
@@ -60,7 +57,7 @@ namespace acommon {
       if (size_ != UINT_MAX) return size_;
       else return size_ = strlen(str_);
     }
-    bool have_size() const {return size_ != UINT_MAX;}
+    bool have_size() {return size_ != UINT_MAX;}
     operator const char * () const {
       return str_;
     }
@@ -71,40 +68,39 @@ namespace acommon {
     const char * str_;
     mutable unsigned int size_;
   };
-
-  typedef const ParmString & ParmStr;
-
-  static inline bool operator== (ParmStr s1, ParmStr s2)
+#ifndef __SUNPRO_CC
+  static inline bool operator== (ParmString s1, ParmString s2)
   {
     if (s1.str() == 0 || s2.str() == 0)
       return s1.str() == s2.str();
     return strcmp(s1,s2) == 0;
   }
-  static inline bool operator== (const char * s1, ParmStr s2)
+#endif
+  static inline bool operator== (const char * s1, ParmString s2)
   {
     if (s1 == 0 || s2.str() == 0)
       return s1 == s2.str();
     return strcmp(s1,s2) == 0;
   }
-  static inline bool operator== (ParmStr s1, const char * s2)
+  static inline bool operator== (ParmString s1, const char * s2)
   {
     if (s1.str() == 0 || s2 == 0)
       return s1.str() == s2;
     return strcmp(s1,s2) == 0;
   }
-  static inline bool operator!= (ParmStr s1, ParmStr s2)
+  static inline bool operator!= (ParmString s1, ParmString s2)
   {
     if (s1.str() == 0 || s2.str() == 0)
       return s1.str() != s2.str();
     return strcmp(s1,s2) != 0;
   }
-  static inline bool operator!= (const char * s1, ParmStr s2)
+  static inline bool operator!= (const char * s1, ParmString s2)
   {
     if (s1 == 0 || s2.str() == 0)
       return s1 != s2.str();
     return strcmp(s1,s2) != 0;
   }
-  static inline bool operator!= (ParmStr s1, const char * s2)
+  static inline bool operator!= (ParmString s1, const char * s2)
   {
     if (s1.str() == 0 || s2 == 0)
       return s1.str() != s2;
