@@ -44,11 +44,6 @@ namespace aspeller {
   using acommon::CheckInfo;
   struct GuessInfo;
 
-  struct CheckList;
-  CheckList * new_check_list();
-  void delete_check_list(CheckList *);
-  CheckInfo * check_list_data(CheckList *);
-
   struct LookupInfo;
   struct AffEntry;
   struct PfxEntry;
@@ -96,7 +91,7 @@ namespace aspeller {
     bool suffix_check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *,
                       int sfxopts, AffEntry* ppfx) const;
 
-    void munch(ParmString word, CheckList *) const;
+    void munch(ParmString word, GuessInfo *) const;
 
     // None of the expand methods reset the objstack
 
@@ -110,7 +105,8 @@ namespace aspeller {
     }
     WordAff * expand_suffix(ParmString word, const unsigned char * new_aff,
                             ObjStack &, int limit = INT_MAX,
-                            unsigned char * new_aff = 0, WordAff * * * l = 0) const;
+                            unsigned char * new_aff = 0, WordAff * * * l = 0,
+                            ParmString orig_word = 0) const;
     
   private:
     PosibErr<void> parse_file(const char * affpath, Conv &);
