@@ -405,11 +405,13 @@ namespace aspeller {
   try_again:
     const char * word = word0;
     const char * inlist = inlist0;
-    if (*word == *inlist || *word == lang->to_title(*inlist)) ++word, ++inlist;
-    else                                                      goto try_upper;
+    if (begin) {
+      if (*word == *inlist || *word == lang->to_title(*inlist)) ++word, ++inlist;
+      else                                                      goto try_upper;
+    }
     while (*word && *inlist && *word == *inlist) ++word, ++inlist;
     if (*inlist) goto try_upper;
-    if (lang->special(*word).end) ++word;
+    if (end && lang->special(*word).end) ++word;
     if (*word) goto try_upper;
     return true;
   try_upper:
@@ -417,48 +419,53 @@ namespace aspeller {
     inlist = inlist0;
     while (*word && *inlist && *word == lang->to_upper(*inlist)) ++word, ++inlist;
     if (*inlist) goto fail;
-    if (lang->special(*word).end) ++word;
+    if (end && lang->special(*word).end) ++word;
     if (*word) goto fail;
     return true;
   fail:
-    if (lang->special(*word0).begin) {++word0; goto try_again;}
+    if (begin && lang->special(*word0).begin) {++word0; goto try_again;}
     return false;
   }
 
+  static PosibErrBase invalid_word_e(const Language & l,
+                                     ParmString word,
+                                     const char * msg,
+                                     char chr = 0)
+  {
+    char m[200];
+    if (chr) {
+      snprintf(m, 200, msg, MsgConv(l)(chr), l.to_uni(chr));
+      msg = m;
+    }
+    return make_err(invalid_word, MsgConv(l)(word), msg);
+  }
+
   PosibErr<void> check_if_valid(const Language & l, ParmString word) {
     if (*word == '\0') 
-      return make_err(invalid_word, MsgConv(l)(word), _("Empty string."));
+      return invalid_word_e(l, word, _("Empty string."));
     const char * i = word;
     if (!l.is_alpha(*i)) {
-      if (!l.special(*i).begin) {
-        char m[70];
-        snprintf(m, 70, 
-                 _("The character '%s' may not appear at the beginning of a word."),
-                 MsgConv(l)(*i));
-        return make_err(invalid_word, MsgConv(l)(word), m);
-      } else if (!l.is_alpha(*(i+1))) {
-	return make_err(invalid_word, MsgConv(l)(word), _("Does not contain any letters."));
-      }
+      if (!l.special(*i).begin)
+        return invalid_word_e(l, word, _("The character '%s' (U+%02X) may not appear at the beginning of a word."), *i);
+      else if (!l.is_alpha(*(i+1)))
+        return invalid_word_e(l, word, _("The character '%s' (U+%02X) must be followed by an alphabetic character."), *i);
+      else if (!*(i+1))
+        return invalid_word_e(l, word, _("Does not contain any alphabetic characters."));
     }
     for (;*(i+1) != '\0'; ++i) { 
       if (!l.is_alpha(*i)) {
-	if (!l.special(*i).middle) {
-          char m[70];
-          snprintf(m, 70, 
-                   _("The character '%s' may not appear in the middle of a word."),
-                   MsgConv(l)(*i));
-          return make_err(invalid_word, MsgConv(l)(word), m);
-        }
+        if (!l.special(*i).middle)
+          return invalid_word_e(l, word, _("The character '%s' (U+%02X) may not appear in the middle of a word."), *i);
+        else if (!l.is_alpha(*(i+1)))
+          return invalid_word_e(l, word, _("The character '%s' (U+%02X) must be followed by an alphabetic character."), *i);
       }
     }
     if (!l.is_alpha(*i)) {
-      if (!l.special(*i).end) {
-        char m[70];
-        snprintf(m, 70, 
-                 _("The character '%s' may not appear at the end of a word."),
-                 MsgConv(l)(*i));
-        return make_err(invalid_word, MsgConv(l)(word), m);
-      }
+      if (*i == '\r')
+        return invalid_word_e(l, word, _("The character '\\r' (U+0D) may not appear at the end of a word. " 
+                                         "This probably means means that the file is using MS-DOS EOL instead of Unix EOL. "), *i);
+      if (!l.special(*i).end)
+        return invalid_word_e(l, word, _("The character '%s' (U+%02X) may not appear at the end of a word."), *i);
     }
     return no_err;
   }
