@@ -287,7 +287,7 @@ namespace aspeller {
 
     bool have_affix() const {return affix_;}
 
-    void munch(ParmString word, CheckList * cl) const {affix_->munch(word, cl);}
+    void munch(ParmString word, GuessInfo * cl) const {affix_->munch(word, cl);}
 
     WordAff * expand(ParmString word, ParmString aff, 
                      ObjStack & buf, int limit = INT_MAX) const {
@@ -354,7 +354,7 @@ namespace aspeller {
     operator bool () const {return lang;}
     int operator() (const char * a, const char * b) const
     { 
-      char x,y;
+      char x, y;
       for (;;)
       {
         while (x = lang->to_clean(*a++), !x);
@@ -396,10 +396,12 @@ namespace aspeller {
   struct SensitiveCompare {
     const Language * lang;
     bool case_insensitive;
-    bool ignore_accents;
+    bool ignore_accents;    
+    bool begin;
+    bool end;
     SensitiveCompare(const Language * l = 0) 
-      : lang(l), case_insensitive(false)
-      , ignore_accents(false) {}
+      : lang(l), case_insensitive(false), ignore_accents(false),
+        begin(true), end(true) {}
     bool operator() (const char * word, const char * inlist) const;
   };
 
@@ -414,7 +416,6 @@ namespace aspeller {
   PosibErr<Language *> new_language(const Config &, ParmString lang = 0);
 
   PosibErr<void> open_affix_file(const Config &, FStream & o);
-  
 }
 
 
