@@ -14,10 +14,6 @@
 
 // data block laid out as follows:
 //
-// The lower 4 bits are used for word info.  The upper 4 bits can be used to store 
-//   info on additional info.
-// * have affix flags
-// * have compound category flags
 // Words:
 //   (<8 bit frequency><8 bit: flags><8 bit: offset>
 //      <8 bit: word size><word><null>
@@ -25,11 +21,14 @@
 // Words with soundslike:
 //   (<8 bit: offset to next item><8 bit: soundslike size><soundslike>
 //      <words>)+
-// An extra <null> is inserted at the end of each "group".
-//
-
-#include <map>
+// Flags are mapped as follows:
+//   bits 0-3: word info
+//   bit    4: duplicate flag
+//   bit    5: <unused>
+//   bit    6: have affix info
+//   bit    7: have compound info
 
+#include <utility>
 using std::pair;
 
 #include <string.h>
@@ -89,16 +88,6 @@ static inline void mmap_free(char * block, unsigned int size)
   munmap(block, size);
 }
 
-// static inline size_t page_size() 
-// {
-// #ifdef _SC_PAGESIZE
-//  /* BSDi does not expose this limit via the sysconf function */
-//   return sysconf (_SC_PAGESIZE);
-// #else
-//   return getpagesize ();
-// #endif
-//}
-
 #else
 
 static inline char * mmap_open(unsigned int, 
@@ -113,15 +102,16 @@ static inline void mmap_free(char *, unsigned int)
   abort();
 }
 
-// static inline size_t page_size() 
-// {
-//   return 1024;
-// }
-
 #endif
 
-static byte HAVE_AFFIX_FLAGS = 1 << 7;
-static byte HAVE_CATEGORY_FLAGS = 1 << 6;
+static byte HAVE_AFFIX_FLAG = 1 << 7;
+static byte HAVE_CATEGORY_FLAG = 1 << 6;
+
+static byte DUPLICATE_FLAG = 1 << 4;
+// this flag is set when there is is more than one word for a
+// particulear "clean" word such as "jello" "Jello".  It is set on all
+// but the last word of the group.  Ie, if it is set than the next
+// word when converted to its "clean" form equals the same value.
 
 static byte WORD_INFO_MASK = 0x0F;
 
@@ -159,7 +149,7 @@ static inline const char * get_sl_words_end(const char * d) {
 
 static inline const char * get_affix(const char * d) {
   int word_size = get_word_size(d);
-  if (get_flags(d) & HAVE_AFFIX_FLAGS) 
+  if (get_flags(d) & HAVE_AFFIX_FLAG) 
     return d + word_size +  1;
   else
     return d + word_size;
@@ -167,14 +157,18 @@ static inline const char * get_affix(const char * d) {
 
 static inline const char * get_category(const char * d) {
   int word_size = get_word_size(d);
-  if (get_flags(d) & HAVE_AFFIX_FLAGS & HAVE_CATEGORY_FLAGS) 
+  if (get_flags(d) & HAVE_AFFIX_FLAG & HAVE_CATEGORY_FLAG) 
     return d + strlen(d + word_size +  1) + 1;
-  else if (get_flags(d) & HAVE_CATEGORY_FLAGS)
+  else if (get_flags(d) & HAVE_CATEGORY_FLAG)
     return d + word_size +  1;
   else
     return d + word_size;
 }
 
+static inline bool duplicate_flag(const char * d) {
+  return get_flags(d) & DUPLICATE_FLAG;
+}
+
 namespace {
 
   using namespace aspeller;
@@ -193,7 +187,7 @@ namespace {
   
   class ReadOnlyDict : public Dictionary
   {
-      
+
   public: //but don't use
 
     struct WordLookupParms {
@@ -202,9 +196,8 @@ namespace {
       typedef BlockVector<const u32int> Vector;
       typedef u32int                    Value;
       typedef const char *              Key;
-      static const bool is_multi = true;
-      Key key(Value v) const {assert (v != u32int_max);
-				return block_begin + v;}
+      static const bool is_multi = false;
+      Key key(Value v) const {return block_begin + v;}
       InsensitiveHash  hash;
       InsensitiveEqual equal;
       bool is_nonexistent(Value v) const {return v == u32int_max;}
@@ -221,13 +214,13 @@ namespace {
     const Jump * jump2;
     WordLookup       word_lookup;
     const char *     word_block;
+    const char *     first_word;
     
     ReadOnlyDict(const ReadOnlyDict&);
     ReadOnlyDict& operator= (const ReadOnlyDict&);
 
-    struct ElementsParms;
+    struct Elements;
     struct SoundslikeElements;
-    struct CleanElements;
 
   public:
     WordEntryEnumeration * detailed_elements() const;
@@ -249,10 +242,9 @@ namespace {
       }
     }
     
-    PosibErr<void> load(ParmString, Config &, LocalDictList *, 
-                        SpellerImpl *, const LocalDictInfo *);
+    PosibErr<void> load(ParmString, Config &, DictList *, SpellerImpl *);
 
-    bool lookup(ParmString word, WordEntry &, const SensitiveCompare &) const;
+    bool lookup(ParmString word, const SensitiveCompare *, WordEntry &) const;
 
     bool clean_lookup(ParmString, WordEntry &) const;
 
@@ -274,22 +266,27 @@ namespace {
   //  
   //
 
-  struct ReadOnlyDict::ElementsParms {
-    typedef WordEntry *                Value;
-    typedef WordLookup::const_iterator Iterator; 
-    const ReadOnlyDict * ws;
-    WordEntry data;
-    ElementsParms(const ReadOnlyDict * w) : ws(w) {}
-    bool endf(const Iterator & i) const {return i.at_end();}
-    Value end_state() const {return 0;}
-    WordEntry * deref(const Iterator & i) {
-      convert(ws->word_block + *i, data);
-      return & data;
+  struct ReadOnlyDict::Elements : public WordEntryEnumeration 
+  {
+    const char * w;
+    WordEntry wi;
+    Elements(const char * w0) : w(w0) {wi.what = WordEntry::Word;}
+    WordEntry * next() {
+      if (get_offset(w) == 0) w += 2; // FIXME: This needs to be 3
+                                      //        when freq info is used
+      if (get_offset(w) == 0) return 0;
+      convert(w, wi);
+      w = get_next(w);
+      return &wi;
     }
+    bool at_end() const {return get_offset(w) == 0;}
+    WordEntryEnumeration * clone() const {return new Elements(*this);}
+    void assign (const WordEntryEnumeration * other) {
+      *this = *static_cast<const Elements *>(other);}
   };
 
   WordEntryEnumeration * ReadOnlyDict::detailed_elements() const {
-    return new MakeEnumeration<ElementsParms>(word_lookup.begin(), ElementsParms(this));
+    return new Elements(first_word);
   }
 
   ReadOnlyDict::Size ReadOnlyDict::size() const {
@@ -300,7 +297,7 @@ namespace {
     return word_lookup.empty();
   }
 
-  static const char * const cur_check_word = "aspell default speller rowl 1.9";
+  static const char * const cur_check_word = "aspell default speller rowl 1.10";
 
   struct DataHead {
     // all sizes except the last four must to divisible by:
@@ -325,6 +322,8 @@ namespace {
     u32int soundslike_name_size;
     u32int soundslike_version_size;
 
+    u32int first_word_offset; // from word block
+
     byte affix_info; // 0 = none, 1 = partially expanded, 2 = full
     byte invisible_soundslike;
     byte soundslike_root_only;
@@ -333,8 +332,7 @@ namespace {
   };
 
   PosibErr<void> ReadOnlyDict::load(ParmString f0, Config & config, 
-                                    LocalDictList *, 
-                                    SpellerImpl *, const LocalDictInfo *)
+                                    DictList *, SpellerImpl *)
   {
     set_file_name(f0);
     const char * fn = file_name();
@@ -412,6 +410,7 @@ namespace {
     }
 
     word_block = block + data_head.word_offset;
+    first_word = word_block + data_head.first_word_offset;
 
     word_lookup.parms().block_begin = word_block;
     word_lookup.parms().hash .lang     = lang();
@@ -424,18 +423,49 @@ namespace {
     return no_err;
   }
 
-  bool ReadOnlyDict::lookup(ParmString word, WordEntry & o,
-                            const SensitiveCompare & c) const 
+  void lookup_adv(WordEntry * wi);
+
+  static inline void prep_next(WordEntry * wi, 
+                               const char * w,
+                               const SensitiveCompare * c, 
+                               const char * orig) 
+  {
+  loop:
+    if (!duplicate_flag(w)) return;
+    w = get_next(w);
+    if (!(*c)(orig, w)) goto loop;
+    wi->intr[0] = (void *)w;
+    wi->intr[1] = (void *)c;
+    wi->intr[2] = (void *)orig;
+    wi->adv_ = lookup_adv;
+  }
+
+  void lookup_adv(WordEntry * wi) 
+  {
+    const char * w = (const char *)wi->intr[0];
+    const SensitiveCompare * c = (const SensitiveCompare *)wi->intr[1];
+    const char * orig = (const char *)wi->intr[2];
+    wi->word = w;
+    wi->adv_ = 0;
+    prep_next(wi, w, c, orig);
+  }
+
+  bool ReadOnlyDict::lookup(ParmString word, const SensitiveCompare * c,
+                            WordEntry & o) const 
   {
     o.clear();
-    o.what = WordEntry::Word;
-    WordLookup::ConstFindIterator i = word_lookup.multi_find(word);
-    for (; !i.at_end(); i.adv()) {
-      const char * w = word_block + i.deref();
-      if (c(word, w)) {
+    WordLookup::const_iterator i = word_lookup.find(word);
+    if (i == word_lookup.end()) return false;
+    const char * w = word_block + *i;
+    for (;;) {
+      if ((*c)(word, w)) {
+        o.what = WordEntry::Word;
         convert(w,o);
+        prep_next(&o, w, c, word);
         return true;
       }
+      if (!duplicate_flag(w)) break;
+      w = get_next(w);
     }
     return false;
   }
@@ -533,34 +563,9 @@ namespace {
     return &data;
   }
 
-  struct ReadOnlyDict::CleanElements : public SoundslikeEnumeration
-  {
-    WordEntry data;
-    const char * cur;
-
-    WordEntry * next(int stopped_at);
-
-    CleanElements(const ReadOnlyDict * o)
-      : cur(o->word_block + 3) {data.what = WordEntry::Word;}
-  };
-
-  WordEntry * ReadOnlyDict::CleanElements::next(int) {
-
-    const char * tmp = cur;
-    cur = get_next(cur);
-    if (cur == tmp) return 0;
-    data.intr[0] = (void *)tmp;
-    convert(tmp, data);
-    return &data;
-
-  }
-
   SoundslikeEnumeration * ReadOnlyDict::soundslike_elements() const {
 
-    if (jump1)
-      return new SoundslikeElements(this);
-    else
-      return new CleanElements(this);
+    return new SoundslikeElements(this);
 
   }
     
@@ -574,18 +579,23 @@ namespace {
     if (cur >= end) w->adv_ = 0;
   }
 
-  //static 
-  //void ReadOnlyDict::clean_next(WordEntry * w) {}
-
-  bool ReadOnlyDict::clean_lookup(ParmString sl, WordEntry & w) const
+  static void clean_lookup_adv(WordEntry * wi) 
   {
-    w.clear();
-    WordLookup::ConstFindIterator i = word_lookup.multi_find(sl);
-    if (i.at_end()) return false;
-    convert(word_block + i.deref(), w);
+    const char * w = wi->word;
+    w = get_next(w);
+    convert(w,*wi);
+    if (!duplicate_flag(w)) wi->adv_ = 0;
+  }
+
+  bool ReadOnlyDict::clean_lookup(ParmString sl, WordEntry & o) const
+  {
+    o.clear();
+    WordLookup::const_iterator i = word_lookup.find(sl);
+    if (i == word_lookup.end()) return false;
+    const char * w = word_block + *i;
+    convert(w, o);
+    if (duplicate_flag(w)) o.adv_ = clean_lookup_adv;
     return true;
-    // FIXME deal with multiple words 
-    //   the lookup should point to the first one of the kind ....
   }
     
   bool ReadOnlyDict::soundslike_lookup(const WordEntry & s, WordEntry & w) const 
@@ -663,9 +673,13 @@ namespace {
   };
 
   struct SoundslikeLess {
+    InsensitiveCompare icomp;
+    SoundslikeLess(const Language * l) : icomp(l) {}
     bool operator() (WordData * x, WordData * y) const {
       int res = strcmp(x->sl, y->sl);
       if (res != 0) return res < 0;
+      res = icomp(x->word, y->word);
+      if (res != 0) return res < 0;
       return strcmp(x->word, y->word) < 0;
     }
   };
@@ -676,9 +690,8 @@ namespace {
     typedef Vector<u32int>      Vector;
     typedef u32int              Value;
     typedef const char *        Key;
-    static const bool is_multi = true;
-    Key key(Value v) const {assert (v != u32int_max);
-                            return block_begin + v;}
+    static const bool is_multi = false;
+    Key key(Value v) const {return block_begin + v;}
     InsensitiveHash  hash;
     InsensitiveEqual equal;
     bool is_nonexistent(Value v) const {return v == u32int_max;}
@@ -754,7 +767,7 @@ namespace {
     data_head.invisible_soundslike = invisible_soundslike;
     data_head.soundslike_root_only = affix_compress  && !partially_expand ? 1 : 0;
 
-//#if 0
+#if 0
     CERR.printl("FLAGS:  ");
     if (full_soundslike) CERR.printl("  full soundslike");
     if (invisible_soundslike) CERR.printl("  invisible soundslike");
@@ -762,11 +775,12 @@ namespace {
     if (affix_compress) CERR.printl("  affix compress");
     if (partially_expand) CERR.printl("  partially expand");
     CERR.printl("---");
-//#endif
+#endif
     
     String temp;
 
     int num_entries = 0;
+    int uniq_entries = 0;
     
     ObjStack buf(16*1024);
     String sl_buf;
@@ -867,7 +881,7 @@ namespace {
           z += s + 1;
 
           if (aff_size > 0) {
-            b->flags |= HAVE_AFFIX_FLAGS;
+            b->flags |= HAVE_AFFIX_FLAG;
             b->aff = z;
             memcpy(z, p->aff, aff_size + 1);
             z += aff_size + 1;
@@ -882,7 +896,6 @@ namespace {
             b->sl = b->word;
           }
 
-          ++num_entries;
         }
       }
       delete els;
@@ -892,18 +905,19 @@ namespace {
     // sort WordData linked list based on (sl, word)
     //
 
-    first = sort(first, SoundslikeLess());
+    first = sort(first, SoundslikeLess(&lang));
 
     //
     // duplicate check
     // 
     WordData * prev = first;
     WordData * cur = first->next;
+    InsensitiveEqual ieq(&lang);
     while (cur) {
       if (strcmp(prev->word, cur->word) == 0) {
         // merge affix info if necessary
         if (!prev->aff && cur->aff) {
-          prev->flags |= HAVE_AFFIX_FLAGS;
+          prev->flags |= HAVE_AFFIX_FLAG;
           prev->aff = cur->aff;
           prev->data_size += strlen(prev->aff) + 1;
         } else if (prev->aff && cur->aff) {
@@ -923,6 +937,9 @@ namespace {
         }
         prev->next = cur->next;
       } else {
+        if (ieq(prev->word, cur->word)) prev->flags |= DUPLICATE_FLAG;
+        else ++uniq_entries;
+        ++num_entries;
         prev = cur;
       }
       cur = cur->next;
@@ -970,8 +987,9 @@ namespace {
     data.write32(0); // to avoid nasty special cases
     unsigned int prev_pos = data.size();
     data.write32(0);
+    unsigned prev_w_pos = data.size();
 
-    WordLookup lookup(num_entries * 4 / 5);
+    WordLookup lookup;
     lookup.parms().block_begin = data.begin();
     lookup.parms().hash .lang     = &lang;
     lookup.parms().equal.cmp.lang = &lang;
@@ -1030,6 +1048,7 @@ namespace {
       if (invisible_soundslike) {
         
         unsigned pos = data.size();
+        prev_w_pos = data.size();
         data.write(p->word, p->word_size + 1);
         if (p->aff) data.write(p->aff, p->data_size - p->word_size - 1);
         lookup.insert(pos);
@@ -1042,23 +1061,23 @@ namespace {
 
         // write all word entries with the same soundslike
 
-        const char * prev;
-
         do {
           data.write(p->flags);
           data.write(p->data_size + 3);
           data.write(p->word_size);
 
           unsigned pos = data.size();
+          data[prev_w_pos - NEXT_O] = (byte)(pos - prev_w_pos);
           data.write(p->word, p->word_size + 1);
           if (p->aff) data.write(p->aff, p->data_size - p->word_size - 1);
           lookup.insert(pos);
 
-          prev = p->sl;
+          prev_w_pos = pos;
+          prev_sl = p->sl;
 
           p = p->next;
 
-        } while (p && prev == p->sl); // yes I really mean to use pointer compare here
+        } while (p && prev_sl == p->sl); // yes I really mean to use pointer compare here
       }
     }
 
@@ -1067,7 +1086,6 @@ namespace {
     data.write16(0);
     data.write16(0);
     data[prev_pos - NEXT_O] |= (byte)(data.size() - prev_pos);
-    data.at32(4) = 0;
     
     jump2.push_back(Jump());
     jump1.push_back(Jump());
@@ -1076,7 +1094,18 @@ namespace {
     data.write(0);
     data.write(0);
 
-    data_head.word_count   = lookup.size();
+    if (invisible_soundslike)
+      data_head.first_word_offset = data[4 - NEXT_O] + 4;
+    else
+      data_head.first_word_offset = data[8 - NEXT_O] + 8;
+
+    memset(data.data(), 0, 8);
+    
+    //CERR.printf("%d == %d\n", lookup.size(), uniq_entries);
+    //assert(lookup.size() == uniq_entries);
+    lookup.resize(lookup.size() * 5 / 4);
+
+    data_head.word_count   = num_entries;
     data_head.word_buckets = lookup.bucket_count();
 
     FStream out;
