@@ -133,7 +133,7 @@ namespace aspeller {
     WS::const_iterator i   = check_ws.begin();
     WS::const_iterator end = check_ws.end();
     do {
-      if (i->dict->lookup(w, w0, i->compare)) return true;
+      if ((*i)->lookup(w, &s_cmp, w0)) return true;
       ++i;
     } while (i != end);
     return false;
@@ -195,6 +195,7 @@ namespace aspeller {
         is_title = lang_->case_pattern(word) == FirstUpper ? Yes : No;
       *i = t;
       if (check(i, word_end, is_title == Yes, run_together_limit - 1, ci + 1, 0)) {
+        ci->compound = true;
         ci->next = ci + 1;
         return true;
       }
@@ -426,15 +427,14 @@ namespace aspeller {
 
   SpellerImpl::SpellerImpl() 
     : Speller(0) /* FIXME */, ignore_repl(true), 
-      dicts_(0), personal_(0), session_(0), repl_(0), main_(0),
-      guess_info(7)
+      dicts_(0), personal_(0), session_(0), repl_(0), main_(0)
   {}
 
-  inline void add_dicts(SpellerImpl * sp, LocalDictList & d)
+  inline void add_dicts(SpellerImpl * sp, DictList & d)
   {
     for (;!d.empty(); d.pop())
     {
-      if (!sp->locate(d.last().dict->id())) {
+      if (!sp->locate(d.last()->id())) {
         sp->add_dict(new SpellerDict(d.last()));
       }
     }
@@ -447,17 +447,28 @@ namespace aspeller {
     ignore_repl = config_->retrieve_bool("ignore-repl");
     ignore_count = config_->retrieve_int("ignore");
 
-    LocalDict res;
-    LocalDictList to_add;
-    RET_ON_ERR(add_data_set(config_->retrieve("master-path"), *config_, res, &to_add, this));
+    DictList to_add;
+    RET_ON_ERR(add_data_set(config_->retrieve("master-path"), *config_, &to_add, this));
     add_dicts(this, to_add);
 
+    s_cmp.lang = lang_;
+
+    s_cmp_begin.lang = lang_;
+    s_cmp_begin.end = false;
+
+    s_cmp_middle.lang = lang_;
+    s_cmp_middle.begin = false;
+    s_cmp_middle.end   = false;
+
+    s_cmp_end.lang = lang_;
+    s_cmp_end.begin = false;
+
     StringList extra_dicts;
     config_->retrieve_list("extra-dicts", &extra_dicts);
     StringListEnumeration els = extra_dicts.elements_obj();
     const char * dict_name;
     while ( (dict_name = els.next()) != 0) {
-      RET_ON_ERR(add_data_set(dict_name,*config_, res, &to_add, this));
+      RET_ON_ERR(add_data_set(dict_name,*config_, &to_add, this));
       add_dicts(this, to_add);
     }
 
@@ -472,7 +483,7 @@ namespace aspeller {
         temp->set_check_lang(lang_name(), *config_);
       else if (pe.has_err())
         return pe;
-      add_dict(new SpellerDict(temp, lang_, *config_, personal_id));
+      add_dict(new SpellerDict(temp, *config_, personal_id));
     }
     
     if (use_other_dicts && !session_)
@@ -480,7 +491,7 @@ namespace aspeller {
       Dictionary * temp;
       temp = new_default_writable_dict();
       temp->set_check_lang(lang_name(), *config_);
-      add_dict(new SpellerDict(temp, lang_, *config_, session_id));
+      add_dict(new SpellerDict(temp, *config_, session_id));
     }
      
     if (use_other_dicts && !repl_)
@@ -491,7 +502,7 @@ namespace aspeller {
         temp->set_check_lang(lang_name(), *config_);
       else if (pe.has_err())
         return pe;
-      add_dict(new SpellerDict(temp, lang_, *config_, personal_repl_id));
+      add_dict(new SpellerDict(temp, *config_, personal_repl_id));
     }
 
     const char * sys_enc = lang_->charmap();
@@ -557,25 +568,21 @@ namespace aspeller {
 
       ti = &typeid(*cur->dict);
 
-      WSInfo inf;
-      inf.dict = cur->dict;
-      inf.set(*cur);
-
       if (cur->use_to_check) {
-        check_ws.push_back(inf);
-        if (inf.dict->affix_compressed) affix_ws.push_back(inf);
+        check_ws.push_back(cur->dict);
+        if (cur->dict->affix_compressed) affix_ws.push_back(cur->dict);
       }
       if (cur->use_to_suggest) {
-        suggest_ws.push_back(inf);
-        if (inf.dict->affix_compressed) suggest_affix_ws.push_back(inf);
+        suggest_ws.push_back(cur->dict);
+        if (cur->dict->affix_compressed) suggest_affix_ws.push_back(cur->dict);
       }
     }
-    fast_scan   = suggest_ws.front().dict->fast_scan;
-    fast_lookup = suggest_ws.front().dict->fast_lookup;
+    fast_scan   = suggest_ws.front()->fast_scan;
+    fast_lookup = suggest_ws.front()->fast_lookup;
     have_soundslike = lang_->have_soundslike();
     have_repl = lang_->have_repl();
-    invisible_soundslike = suggest_ws.front().dict->invisible_soundslike;
-    soundslike_root_only = suggest_ws.front().dict->soundslike_root_only;
+    invisible_soundslike = suggest_ws.front()->invisible_soundslike;
+    soundslike_root_only = suggest_ws.front()->soundslike_root_only;
     affix_compress = !affix_ws.empty();
 
     //
@@ -629,8 +636,8 @@ namespace aspeller {
   //
   //
 
-  SpellerDict::SpellerDict(LocalDict & d) 
-    : LocalDict(d), special_id(none_id), next(0) 
+  SpellerDict::SpellerDict(Dict * d) 
+    : dict(d), special_id(none_id), next(0) 
   {
     switch (dict->basic_type) {
     case Dict::basic_dict:
@@ -648,10 +655,9 @@ namespace aspeller {
     save_on_saveall = false;
   }
 
-  SpellerDict::SpellerDict(Dict * w, const Language * l, const Config & c, SpecialId id)
+  SpellerDict::SpellerDict(Dict * w, const Config & c, SpecialId id)
     : next(0) 
   {
-    set(l, c);
     dict = w;
     special_id = id;
     switch (id) {
