@@ -76,6 +76,7 @@ void filter();
 void list();
 void dicts();
 
+void clean();
 void master();
 void personal();
 void repl();
@@ -145,13 +146,14 @@ struct PossibleOption {
 #define ISPELL_COMP(abrv,num)         {"",abrv,num,false}
 
 const PossibleOption possible_options[] = {
-  OPTION("master",           'd',  1),
-  OPTION("personal",         'p',  1),
-  OPTION("ignore",            'W', 1),
-  OPTION("backup",           'b' , 0),
-  OPTION("dont-backup",      'x' , 0),
-  OPTION("run-together",     'C',  0),
-  OPTION("dont-run-together",'B',  0),
+  OPTION("master",           'd', 1),
+  OPTION("personal",         'p', 1),
+  OPTION("ignore",           'W', 1),
+  OPTION("lang",             'l', 1),
+  OPTION("backup",           'b', 0),
+  OPTION("dont-backup",      'x', 0),
+  OPTION("run-together",     'C', 0),
+  OPTION("dont-run-together",'B', 0),
   OPTION("guess",            'm', 0),
   OPTION("dont-guess",       'P', 0),
   
@@ -167,8 +169,9 @@ const PossibleOption possible_options[] = {
   COMMAND("munch",     '\0', 0),
   COMMAND("expand",    '\0', 0),
   COMMAND("combine",   '\0', 0),
-  COMMAND("list",      'l', 0),
+  COMMAND("list",      '\0', 0),
   COMMAND("dicts",     '\0', 0),
+  COMMAND("clean",     '\0', 0),
 
   COMMAND("dump",   '\0', 1),
   COMMAND("create", '\0', 1),
@@ -395,6 +398,8 @@ int main (int argc, const char *argv[])
     action = do_create;
   else if (action_str == "merge")
     action = do_merge;
+  else if (action_str == "clean")
+    clean();
   else {
     print_error(_("Unknown Action: %s"),  action_str);
     return 1;
@@ -564,18 +569,32 @@ void print_elements(const AspellWordList * wl) {
   COUT.printf("%u: %s\n", count, line.c_str());
 }
 
+struct StatusFunInf 
+{
+  aspeller::SpellerImpl * real_speller;
+  bool verbose;
+};
+
 void status_fun(void * d, Token, int correct)
 {
-  if (*static_cast<bool *>(d) && correct)
-    COUT.put("*\n");
+  StatusFunInf * p = static_cast<StatusFunInf *>(d);
+  if (p->verbose && correct) {
+    const CheckInfo * ci = p->real_speller->check_info();
+    if (ci->compound)
+      COUT.put("-\n");
+    else if (ci->pre_flag || ci->suf_flag)
+      COUT.printf("+ %s\n", ci->word.str());
+    else
+      COUT.put("*\n");
+  }
 }
 
 DocumentChecker * new_checker(AspellSpeller * speller, 
-			      bool & print_star) 
+			      StatusFunInf & status_fun_inf) 
 {
   EXIT_ON_ERR_SET(new_document_checker(reinterpret_cast<Speller *>(speller)),
 		  StackPtr<DocumentChecker>, checker);
-  checker->set_status_fun(status_fun, &print_star);
+  checker->set_status_fun(status_fun, &status_fun_inf);
   return checker.release();
 }
 
@@ -616,8 +635,11 @@ void pipe()
   if (do_time)
     COUT << _("Time to load word list: ")
          << (clock() - start)/(double)CLOCKS_PER_SEC << "\n";
-  bool print_star = true;
-  StackPtr<DocumentChecker> checker(new_checker(speller, print_star));
+  StatusFunInf status_fun_inf;
+  status_fun_inf.real_speller = real_speller;
+  bool & print_star = status_fun_inf.verbose;
+  print_star = true;
+  StackPtr<DocumentChecker> checker(new_checker(speller, status_fun_inf));
   int c;
   const char * w;
   CharVector buf;
@@ -671,13 +693,13 @@ void pipe()
 	config->replace("mode", "tex");
       reload_filters(real_speller);
       checker.del();
-      checker = new_checker(speller, print_star);
+      checker = new_checker(speller, status_fun_inf);
       break;
     case '-':
       config->remove("filter");
       reload_filters(real_speller);
       checker.del();
-      checker = new_checker(speller, print_star);
+      checker = new_checker(speller, status_fun_inf);
       break;
     case '~':
       break;
@@ -1265,6 +1287,98 @@ void print_ver () {
 // module
 //
 
+///////////////////////////
+//
+// clean
+//
+
+void clean()
+{
+  using namespace aspeller;
+
+  bool strict = args.size() != 0 && args[0] == "strict";
+  
+  StackPtr<Config> config(new_basic_config());
+  EXIT_ON_ERR(config->read_in_settings(options));
+
+  CachePtr<Language> lang;
+  find_language(*config);
+  PosibErr<Language *> res = new_language(*config);
+  if (!res) {print_error(res.get_err()->mesg); exit(1);}
+  lang.reset(res.data);
+  ConvEC iconv; 
+  Conv oconv, oconv2;
+  bool have_affix = lang->have_affix();
+  if (!config->have("norm-strict"))
+    config->replace("norm-strict", "true");
+  if (config->have("encoding")) {
+    EXIT_ON_ERR(iconv.setup(*config, config->retrieve("encoding"), lang->charmap(),NormFrom));
+    EXIT_ON_ERR(oconv.setup(*config, lang->charmap(), config->retrieve("encoding"), NormTo));
+    oconv2.setup(*config, lang->charmap(), config->retrieve("encoding"), NormTo);
+  } else {
+    EXIT_ON_ERR(iconv.setup(*config, lang->data_encoding(), lang->charmap(), NormFrom));
+    EXIT_ON_ERR(oconv.setup(*config, lang->charmap(), lang->data_encoding(), NormTo));
+    oconv2.setup(*config, lang->charmap(), lang->data_encoding(), NormTo);
+  }
+  
+  String data;
+  char * str;
+  char * str_end;
+  char * aff = "";
+  char brk[3] = " ";
+  if (!lang->special('-').any) brk[1] = '-';
+  while (CIN.getline(data)) {
+    data.ensure_null_end();
+    PosibErr<char *> pe = iconv(data.data(), data.size());
+    if (pe.has_err()) {
+      CERR.printf("Error: %s Skipping string.\n", pe.get_err()->mesg);
+      continue;
+    } else {
+      String * s = pe.data == data.data() ? &data : &iconv.buf;
+      str = s->pbegin();
+      str_end = s->pend();
+    }
+    if (have_affix) {
+      aff = strchr(str, '/');
+      if (aff == 0) {
+        aff = str_end;
+      } else {
+        *aff = '\0';
+        str_end = aff;
+        ++aff;
+      }
+    }
+    if (!strict) {
+      char * s = str;
+      while (s < str_end) {
+        while (s < str_end && !lang->is_alpha(*s) && !lang->special(*s).begin)
+          *s++ = '\0';
+        if (*aff) {
+          s = str_end;
+        } else {
+          s += strcspn(s, brk);
+          *s = '\0';
+        }
+        char * s2 = s - 1;
+        while (s2 >= str && *s2 && !lang->is_alpha(*s2) && !lang->special(*s2).end)
+          *s2-- = '\0';
+      }
+    }
+    for (; str < str_end; str += strlen(str) + 1) {
+      if (!*str || (!strict && !*aff && !str[1])) continue;
+      PosibErrBase pe2 = check_if_valid(*lang, str);
+      if (pe2.has_err()) {
+        CERR.printf("Error: %s Skipping word.\n", pe2.get_err()->mesg);
+      } else {
+        if (*aff) 
+          COUT.printf("%s/%s\n", oconv(str), oconv(aff));
+        else
+          COUT.printl(oconv(str));
+      }
+    }
+  }
+}
+
 ///////////////////////////
 //
 // master
@@ -1288,14 +1402,14 @@ public:
   bool at_end() const {return *in;}
 };
 
-void dump (aspeller::LocalDict lws, Convert * conv) 
+void dump (aspeller::Dict * lws, Convert * conv) 
 {
   using namespace aspeller;
 
-  switch (lws.dict->basic_type) {
+  switch (lws->basic_type) {
   case Dict::basic_dict:
     {
-      Dictionary * ws = static_cast<Dictionary *>(lws.dict);
+      Dictionary * ws = static_cast<Dictionary *>(lws);
       StackPtr<WordEntryEnumeration> els(ws->detailed_elements());
       WordEntry * wi;
       while (wi = els->next(), wi) {
@@ -1306,10 +1420,10 @@ void dump (aspeller::LocalDict lws, Convert * conv)
     break;
   case Dict::multi_dict:
     {
-      StackPtr<DictsEnumeration> els(lws.dict->dictionaries());
-      const LocalDict * ws;
+      StackPtr<DictsEnumeration> els(lws->dictionaries());
+      Dict * ws;
       while (ws = els->next(), ws) 
-	dump (*ws, conv);
+	dump (ws, conv);
     }
     break;
   default:
@@ -1341,9 +1455,8 @@ void master () {
     
   } else if (action == do_dump) {
 
-    LocalDict d;
-    EXIT_ON_ERR(add_data_set(config->retrieve("master-path"), *config, d));
-    StackPtr<Convert> conv(setup_conv(d.dict->lang(), config));
+    EXIT_ON_ERR_SET(add_data_set(config->retrieve("master-path"), *config), Dict *, d);
+    StackPtr<Convert> conv(setup_conv(d->lang(), config));
     dump(d, conv);
   }
 }
@@ -1393,8 +1506,6 @@ void personal () {
     Dictionary * per = new_default_writable_dict();
     per->load(config->retrieve("personal-path"), *config);
     StackPtr<WordEntryEnumeration> els(per->detailed_elements());
-    LocalDictInfo wsi;
-    wsi.set(per->lang(), *config);
     StackPtr<Convert> conv(setup_conv(per->lang(), config));
 
     WordEntry * wi;
@@ -1513,11 +1624,11 @@ void munch()
   Conv iconv(setup_conv(options, lang));
   Conv oconv(setup_conv(lang, options));
   String word;
-  CheckList * cl = new_check_list();
+  GuessInfo gi;
   while (CIN.getline(word)) {
-    lang->munch(iconv(word), cl);
+    lang->munch(iconv(word), &gi);
     COUT << word;
-    for (const aspeller::CheckInfo * ci = check_list_data(cl); ci; ci = ci->next)
+    for (const aspeller::CheckInfo * ci = gi.head; ci; ci = ci->next)
     {
       COUT << ' ' << oconv(ci->word) << '/';
       if (ci->pre_flag != 0) COUT << oconv(static_cast<char>(ci->pre_flag));
@@ -1525,7 +1636,6 @@ void munch()
     }
     COUT << '\n';
   }
-  delete_check_list(cl);
 }
 
 //////////////////////////
@@ -1573,7 +1683,7 @@ void expand()
       WordAff * p = exp_list;
       while (p) {
         COUT << oconv(p->word);
-        if (p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
+        if (limit < INT_MAX && p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
         p = p->next;
         if (p) COUT << ' ';
       }
@@ -1588,7 +1698,7 @@ void expand()
       }
       for (WordAff * p = exp_list; p; p = p->next) {
         COUT << word << ' ' << oconv(p->word);
-        if (p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
+        if (limit < INT_MAX && p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
         if (level >= 4) COUT.printf(" %f\n", ratio);
         else COUT << '\n';
       }
@@ -1805,18 +1915,24 @@ void print_help () {
     "                    and help for filters matching <expr> if installed\n"
     "  -c|check <file>  to check a file\n"
     "  -a|pipe          \"ispell -a\" compatibility mode\n"
-    "  -l|list          produce a list of misspelled words from standard input\n"
+    "  list             produce a list of misspelled words from standard input\n"
     "  [dump] config [-e <expr>]  dumps the current configuration to stdout\n"
     "  config [+e <expr>] <key>   prints the current value of an option\n"
     "  soundslike       returns the sounds like equivalent for each word entered\n"
     "  munch            generate possible root words and affixes\n"
     "  expand [1-4]     expands affix flags\n"
+    "  clean [strict]   cleans a word list so that every line is a valid word\n"
     "  filter           passes standard input through filters\n"
     "  -v|version       prints a version line\n"
+    "  conv <from> <to> [<norm-form>]\n"
+    "    converts from one encoding to another\n"
+    "  norm (<norm-map> | <from> <norm-map> <to>) [<norm-form>]\n"
+    "    perform unicode normlization\n"
     "  dump|create|merge master|personal|repl [word list]\n"
     "    dumps, creates or merges a master, personal, or replacement word list.\n"
     "\n"
     "  <expr>           regular expression matching filtername(s) or \"all\"\n"
+    "  <norm-form>      normalization form to use, either none, internal, or strict\n"
     "\n"
     "[options] is any of the following:\n"
     "\n"), VERSION);
